@@ -1,8 +1,8 @@
 ###              Classes of positive-definite matrices
 ###
+### Copyright 2006-2020  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
-### Copyright 2006-2016  The R Core team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ pdMatrix <-
 
 pdMat <-
   function(value = numeric(0), form = NULL, nam = NULL,
-	   data = sys.frame(sys.parent()), pdClass = "pdSymm")
+	   data = parent.frame(), pdClass = "pdSymm")
 {
   if (inherits(value, "pdMat")) {	# nothing to construct
     pdClass <- class(value)
@@ -69,7 +69,7 @@ corMatrix.pdMat <-
 
 pdConstruct.pdMat <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.frame(sys.parent()), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   if (inherits(value, "pdMat")) {	# constructing from another pdMat
     if (length(form) == 0) {
@@ -85,7 +85,7 @@ pdConstruct.pdMat <-
     }
   }
   if (length(value) > 0) {
-    if (inherits(value, "formula") || data.class(value) == "call") {
+    if (inherits(value, "formula") || is.call(value)) {
       ## constructing from a formula
       if (!is.null(form)) {
 	warning("ignoring argument 'form'")
@@ -128,7 +128,7 @@ pdConstruct.pdMat <-
       value <- as.numeric(value)
       attributes(value) <- attributes(object)
       object <- value
-    } else if (data.class(value) == "list") {
+    } else if(is.list(value)) {
       ## constructing from a list of two-sided formulae - nlme case
       if (!is.null(form)) {
 	warning("ignoring argument 'form'")
@@ -265,37 +265,29 @@ coef.pdMat <-
   value
 }
 
-Dim.pdMat <-
-  function(object, ...)
+Dim.pdMat <- function(object, ...)
 {
-  if ((val <- length(Names(object))) > 0) {
-    return(c(val, val))
-  } else if (isInitialized(object)) {
-    return(dim(as.matrix(object)))
-  }
-  stop("cannot access the number of columns of uninitialized objects without names")
+  if ((val <- length(Names(object))) > 0)
+    c(val, val)
+  else if (isInitialized(object))
+    dim(as.matrix(object))
+  else
+    stop("cannot access the number of columns of uninitialized objects without names")
 }
 
 formula.pdMat <-
   function(x, asList, ...) eval(attr(x, "formula"))
 
-isInitialized.pdMat <-
-  function(object)
-{
-  length(object) > 0
-}
+isInitialized.pdMat <- function(object) length(object) > 0
 
-logDet.pdMat <-
-  function(object, ...)
+logDet.pdMat <- function(object, ...)
 {
-  if (!isInitialized(object)) {
+  if (!isInitialized(object))
     stop("cannot extract the log of the determinant from an uninitialized object")
-  }
-  sum(log(svd(pdMatrix(object, factor = TRUE))$d))
+  sum(log(svd.d(pdMatrix(object, factor = TRUE))))
 }
 
-"matrix<-.pdMat" <-
-  function(object, value)
+`matrix<-.pdMat` <- function(object, value)
 {
   value <- as.matrix(value)
   ## check for consistency of dimensions when object is initialized
@@ -311,8 +303,7 @@ Names.pdMat <-
   as.character(attr(object, "Dimnames")[[2]])
 }
 
-"Names<-.pdMat" <-
-  function(object, ..., value)
+`Names<-.pdMat` <- function(object, ..., value)
 {
   if (is.null(value)) {
     attr(object, "Dimnames") <- NULL
@@ -594,7 +585,7 @@ pdSymm <-
 
 pdConstruct.pdSymm <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.frame(sys.parent()), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   val <- NextMethod()
   if (length(val) == 0) {               # uninitialized object
@@ -629,16 +620,14 @@ pdFactor.pdSymm <-
      as.double(object))$Factor
 }
 
-pdMatrix.pdSymm <-
-  function(object, factor = FALSE)
+pdMatrix.pdSymm <- function(object, factor = FALSE)
 {
-  if (!isInitialized(object)) {
+  if (!isInitialized(object))
     stop("cannot extract matrix from an uninitialized object")
-  }
   if (factor) {
     Ncol <- Dim(object)[2]
     value <- array(pdFactor(object), c(Ncol, Ncol), attr(object, "Dimnames"))
-    attr(value, "logDet") <- sum(log(abs(svd(value)$d)))
+    attr(value, "logDet") <- sum(log(abs(svd.d(value))))
     value
   } else {
     NextMethod()
@@ -712,7 +701,7 @@ summary.pdSymm <-
 
 pdLogChol <-
   ## Constructor for the pdLogChol class
-  function(value = numeric(0), form = NULL, nam = NULL, data = sys.parent())
+  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 {
   object <- numeric(0)
   class(object) <- c("pdLogChol", "pdMat")
@@ -723,7 +712,7 @@ pdLogChol <-
 
 pdConstruct.pdLogChol <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.parent(), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   val <- pdConstruct.pdMat(object, value, form, nam, data)
   if (length(val) == 0) {               # uninitialized object
@@ -796,7 +785,7 @@ summary.pdLogChol <-
 
 #pdChol <-
 #  ## Constructor for the pdChol class
-#  function(value = numeric(0), form = NULL, nam = NULL, data = sys.parent())
+#  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 #{
 #  object <- numeric(0)
 #  class(object) <- c("pdChol", "pdMat")
@@ -807,7 +796,7 @@ summary.pdLogChol <-
 
 #pdConstruct.pdChol <-
 #  function(object, value = numeric(0), form = formula(object),
-#	   nam = Names(object), data = sys.parent())
+#	   nam = Names(object), data = parent.frame())
 #{
 #  val <- pdConstruct.pdMat(object, value, form, nam, data)
 #  if (length(val) == 0) {               # uninitialized object
@@ -877,7 +866,7 @@ summary.pdLogChol <-
 
 #pdSpher <-
 #  ## Constructor for the pdSpher class
-#  function(value = numeric(0), form = NULL, nam = NULL, data = sys.parent())
+#  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 #{
 #  object <- numeric(0)
 #  class(object) <- c("pdSpher", "pdMat")
@@ -888,7 +877,7 @@ summary.pdLogChol <-
 
 #pdConstruct.pdSpher <-
 #  function(object, value = numeric(0), form = formula(object),
-#	   nam = Names(object), data = sys.parent())
+#	   nam = Names(object), data = parent.frame())
 #{
 #  val <- pdConstruct.pdMat(object, value, form, nam, data)
 #  if (length(val) == 0) {			# uninitiliazed object
@@ -941,7 +930,7 @@ summary.pdLogChol <-
 
 #pdMatrixLog <-
 #  ## Constructor for the pdMatrixLog class
-#  function(value = numeric(0), form = NULL, nam = NULL, data = sys.parent())
+#  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 #{
 #  object <- numeric(0)
 #  class(object) <- c("pdMatrixLog", "pdMat")
@@ -952,7 +941,7 @@ summary.pdLogChol <-
 
 #pdConstruct.pdMatrixLog <-
 #  function(object, value = numeric(0), form = formula(object),
-#	   nam = Names(object), data = sys.parent())
+#	   nam = Names(object), data = parent.frame())
 #{
 #  val <- pdConstruct.pdMat(object, value, form, nam, data)
 #  if (length(val) == 0) {               # uninitialized object
@@ -1017,7 +1006,7 @@ summary.pdLogChol <-
 
 #pdGivens <-
 #  ## Constructor for the pdGivens class
-#  function(value = numeric(0), form = NULL, nam = NULL, data = sys.parent())
+#  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 #{
 #  object <- numeric(0)
 #  class(object) <- c("pdGivens", "pdMat")
@@ -1028,7 +1017,7 @@ summary.pdLogChol <-
 
 #pdConstruct.pdGivens <-
 #  function(object, value = numeric(0), form = formula(object),
-#	   nam = Names(object), data = sys.parent())
+#	   nam = Names(object), data = parent.frame())
 #{
 #  val <- pdConstruct.pdMat(object, value, form, nam, data)
 #  if (length(val) == 0) {               # uninitiliazed object
@@ -1109,7 +1098,7 @@ summary.pdLogChol <-
 
 pdNatural <-
   ## Constructor for the pdNatural class
-  function(value = numeric(0), form = NULL, nam = NULL, data = sys.frame(sys.parent()))
+  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 {
   object <- numeric(0)
   class(object) <- c("pdNatural", "pdMat")
@@ -1120,7 +1109,7 @@ pdNatural <-
 
 pdConstruct.pdNatural <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.frame(sys.parent()), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   val <- pdConstruct.pdMat(object, value, form, nam, data)
   if (length(val) == 0) {               # uninitiliazed object
@@ -1264,7 +1253,7 @@ summary.pdNatural <-
 
 pdDiag <-
   ## Constructor for the pdDiag class
-  function(value = numeric(0), form = NULL, nam = NULL, data = sys.frame(sys.parent()))
+  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 {
   object <- numeric(0)
   class(object) <- c("pdDiag", "pdMat")
@@ -1289,7 +1278,7 @@ corMatrix.pdDiag <-
 
 pdConstruct.pdDiag <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.frame(sys.parent()), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   val <- NextMethod()
   if (length(val) == 0) {               # uninitiliazed object
@@ -1394,7 +1383,7 @@ summary.pdDiag <-
 
 pdIdent <-
   ## Constructor for the pdIdent class
-  function(value = numeric(0), form = NULL, nam = NULL, data = sys.frame(sys.parent()))
+  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 {
   object <- numeric(0)
   class(object) <- c("pdIdent", "pdMat")
@@ -1424,7 +1413,7 @@ corMatrix.pdIdent <-
 
 pdConstruct.pdIdent <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.frame(sys.parent()), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   val <- NextMethod()
   if (length(val) == 0) {			# uninitialized object
@@ -1533,7 +1522,7 @@ summary.pdIdent <-
 
 pdCompSymm <-
   ## Constructor for the pdCompSymm class
-  function(value = numeric(0), form = NULL, nam = NULL, data = sys.frame(sys.parent()))
+  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame())
 {
   object <- numeric(0)
   class(object) <- c("pdCompSymm", "pdMat")
@@ -1567,7 +1556,7 @@ corMatrix.pdCompSymm <-
 
 pdConstruct.pdCompSymm <-
   function(object, value = numeric(0), form = formula(object),
-	   nam = Names(object), data = sys.frame(sys.parent()), ...)
+	   nam = Names(object), data = parent.frame(), ...)
 {
   val <- NextMethod()
   if (length(val) == 0) {                # uninitialized object
@@ -1691,7 +1680,7 @@ summary.pdCompSymm <-
 
 pdBlocked <-
   ## Constructor for the pdBlocked class
-  function(value = numeric(0), form = NULL, nam = NULL, data = sys.frame(sys.parent()),
+  function(value = numeric(0), form = NULL, nam = NULL, data = parent.frame(),
 	   pdClass = "pdSymm")
 {
   object <- numeric(0)
@@ -1727,14 +1716,15 @@ corMatrix.pdBlocked <-
 
 pdConstruct.pdBlocked <-
   function(object, value = numeric(0), form = formula(object, TRUE),
-	   nam = Names(object, TRUE), data = sys.frame(sys.parent()),
+	   nam = Names(object, TRUE), data = parent.frame(),
 	   pdClass = "pdSymm", ...)
 {
   if (inherits(value, "pdMat")) {	# constructing from another pdMat
     if (inherits(value, "pdBlocked")) {
       if (length(form) == 0) form <- formula(value, TRUE)
       if (length(nam) == 0) nam <- Names(value, TRUE)
-      if (missing(pdClass)) pdClass <- unlist(lapply(value, data.class))
+      if (missing(pdClass)) ## somewhat dubious (why keep all? / order?):
+          pdClass <- unlist(lapply(value, data.class))
     }
     if (isInitialized(value)) {
       return(pdConstruct(object, as.matrix(value), form, nam, data, pdClass))
@@ -1745,18 +1735,14 @@ pdConstruct.pdBlocked <-
   }
   ## checking validity and consistency of form, nam, and pdClass
   if (!is.null(form)) {
-    if (data.class(form) != "list") {
-      stop("'form' must be a list")
-    }
+    if(!is.list(form)) stop("'form' must be a list")
     nF <- length(form)
   } else {
     nF <- 0
   }
 
   if (!is.null(nam)) {
-    if (data.class(nam) != "list") {
-      stop("'nam' must be a list")
-    }
+    if(!is.list(nam)) stop("'nam' must be a list")
     nN <- length(nam)
     if ((nF > 0) && (nN != nF)) {
       stop("'form' and 'nam' have incompatible lengths")
@@ -1792,9 +1778,8 @@ pdConstruct.pdBlocked <-
     ## will first do a null initialization when value is a matrix or numeric
     value <- lapply(vector("list", nB), function(el) numeric(0))
   } else {
-    if (data.class(value) != "list") {
+    if (!is.list(value))
       stop("'object' must be a list when not missing, not a matrix, and not numeric")
-    }
     nO <- length(value)
     if ((nB > 1) && (nB != nO)) {
       stop("arguments imply different number of blocks")
@@ -2123,20 +2108,18 @@ summary.pdBlocked <-
     val <- eval(mCall)
     vNames <- colnames(val)
     auxNames <- lapply(Names(xx, TRUE),
-		       function(el, vN) {
-			 aux <- match(vN, el)
-			 if (any(aux1 <- !is.na(aux))) {
-			   el[aux[aux1]]
-			 }
-		       }, vN = vNames)
-    auxWhich <- !unlist(lapply(auxNames, is.null))
+		       function(el) {
+			 aux <- match(vNames, el)
+			 if(any(ok <- !is.na(aux))) el[aux[ok]] # else NULL
+		       })
+    auxWhich <- !vapply(auxNames, is.null, NA)
     if (sum(auxWhich) == 1) {
-      return(pdConstruct(as.list(xx)[auxWhich][[1]], val))
+	pdConstruct(as.list(xx)[auxWhich][[1]], val)
+    } else {
+	auxNames <- auxNames[auxWhich]
+	auxClass <- vapply(xx, function(el) class(el)[1L], "")[auxWhich]
+	pdConstruct(xx, val, nam = auxNames, form = NULL, pdClass = auxClass)
     }
-    auxNames <- auxNames[auxWhich]
-    auxClass <- unlist(lapply(xx, function(el) class(el)[1]))[auxWhich]
-    return(pdConstruct(xx, val, nam = auxNames, form = NULL,
-		       pdClass = auxClass))
   } else {
     eval(mCall)
   }
