@@ -1,6 +1,6 @@
 ###   Miscellaneous methods that must be defined last in the library
 ###
-### Copyright 2007-2018 The R Core team
+### Copyright 2007-2022  The R Core Team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
 #
@@ -18,30 +18,15 @@
 #  http://www.r-project.org/Licenses/
 #
 
-## Because of the conditional pre-/post 3.3.0 behavior with 'sigma' :
-.onLoad <- function(libname, pkgname) {
-    ## pInfo <- readRDS(attr(packageDescription(pkgname), "file"))
-    ## built.R.ver <- pInfo$Built$R
-    ## First case: does signal Error : object 'sigma' is not exported by 'namespace:stats'
-    ##             so we don't need the following:
-    ## if(getRversion() < "3.3") { ## check that nlme was not *installed* with R >= 3.3.x
-    ##     if(built.R.ver >= "3.3")
-    ##         warning("Package ", dQuote(pkgname), " installed with R version ", built.R.ver,
-    ##     	 " can not safely be used with old R version ", getRversion())
-    ##
-    ## second case:
-    if(getRversion() >= "3.3") {
-        pInfo <- readRDS(attr(packageDescription(pkgname), "file"))
-        built.R.ver <- pInfo$Built$R
-	if(built.R.ver < "3.3")## installed with R < 3.3.x :
-	    warning("Package ", dQuote(pkgname), " installed with old R version ",
-		    built.R.ver, " should not be used with R version ", getRversion(),
-		    "\n  Rather re-install it with this version of R.")
-    }
-}
-
 `%||%` <- function(x, y) if(is.null(x)) y else x
 
+## used in test scripts to switch on additional maintainer checks
+doExtras <- function ()
+{
+    interactive() || nzchar(Sys.getenv("R_nlme_check_extra")) ||
+        identical("true", unname(Sys.getenv("R_PKG_CHECKING_doExtras")))
+}
+
 ## Note that  require( stats )  has already happened ...
 
 comparePred.lme <- comparePred.lmList <- comparePred.gls
@@ -80,16 +65,12 @@ extractAIC.gls <- extractAIC.lme <- function(fit, scale, k = 2, ...)
     c(edf,  -2*res + k * edf)
 }
 
-terms.gls <- function(x, ...) terms(formula(x), ...)
-if(FALSE)## Not needed, because 'lme' object has "terms" attribute:
-    terms.lme <- function(x, ...) terms(formula(x), ...)
+## no longer needed, because gls() and lme() keep the model "terms"
+## terms.gls <- function(x, ...) terms(formula(x), ...)
+## terms.lme <- function(x, ...) terms(formula(x), ...)
 ## end{from MASS}
 
 
-if(getRversion() < "3.3") {
-    sigma <- function(object, ...) UseMethod("sigma")
-}
-
 sigma.gls <- sigma.lme <- function(object, ...) object$sigma
 
 ## also works for "nlsList"
